#################################################################################
# Created       : Martin Foster
# Modified      : 08/28/2001
#################################################################################
#
# Login - Modules part of Webchat designed to simplify login  
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

package Ethereal::Login;					# Package name

use Ethereal::Audit;						# Audit trail
use Ethereal::Crypto;						# Cryptographic
use Ethereal::Option;						# Options handling
use Ethereal::Table;						# Table Creation

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	#####################
	# Data members

        my $class    = shift;					# Reference to self
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter handle

        my $self  = {
		HANDLE => $database,
		CGI    => $cgi,
		PARAM  => $param 
	 };


	#####################
	# Program area

        # Bless $self into designated class
        bless $self, $class;

        # Return self
        return $self;
}



#####################
# Authenthication
#
# This is the primary component of this module, and this would be the 
# authenthication script, which will verify usernames and passwords.

sub Authenthication
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $choice   = shift;					# Choice of the matter

	my $cgi      = $self->{'CGI'};				# Common gateway interface
	my $database = $self->{'HANDLE'};			# Database handle
	my $param    = $self->{'PARAM'};			# Parameter list

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# SQL statement handle 

	my $cookie;						# System cookie
	my $crypt;						# Crypted password
	my $check;						# Check registration	

	my %login;						# Login dbm


	#####################
	# Program area

	# Timeout kill
	if (defined($param->{'MORI'}))
	{
		# Retreive cookie
		$cookie = $cgi->cookie(-name=>'CRYPT');

		# Last attempt at authenthication
		if (defined($cookie))
		{
			# Retreive infromation
			$crypto = new Ethereal::Crypto();
			$crypto->DecryptParam($cgi, $cookie, $param);

			if (defined($param->{'MORI'})) { return 0; }
		}
		else { return 0; }
	}

	# Prepare and execute
	if ($choice == 1)					# Adminstrative
	{ 
		$statement = $data->prepare("SELECT PuppeteerPassword
					FROM Puppeteer 
					WHERE PuppeteerLogin=?
					  AND PuppeteerAdminPrivs='yes'"); 
	}

	if ($choice == 2)					# Supervisory
	{ 
		$statement = $data->prepare("SELECT PuppeteerPassword 
					FROM Puppeteer 
					WHERE PuppeteerLogin=?
					  AND PuppeteerSuperPrivs='yes'");
	}

	elsif ($choice >= 99) 					# Non-Authoritative
	{ 
		$statement = $data->prepare("SELECT PuppeteerPassword 
					FROM Puppeteer 
					WHERE PuppeteerLogin=?"); 
	}
	$statement->execute($param->{'USER'});

	# Retreive and finish
	$crypt = $statement->fetchrow();
	$statement->finish();

	# Compare passwords
	if (defined($crypt))
	{
		# If crypt
		if (defined($param->{'CRYPT'}))
		{
			# Fail on non-match
			if ($param->{'CRYPT'} ne $crypt)
			{ return 0; }
		}

		# If first time
		elsif (defined($param->{'PASS'}))
		{
			# Fail on non-match
			if (crypt($param->{'PASS'}, $crypt) ne $crypt)
			{ return 0; }

			# Otherwise assign crypt
			else
			{ $param->{'CRYPT'} = (crypt($param->{'PASS'}, $crypt)); }
		}

		# Assume fault
		else
		{ return 0; }
	}

	# Assume fault
	else
	{ return 0; }


	# Verify for incomplete registration
	# Prepare and execute
	$statement = $data->prepare("SELECT PuppeteerLogin 
			FROM PuppeteerRegistration
			WHERE PuppeteerLogin=?"); 
	$statement->execute($param->{'USER'});

	# Retreive and finish
	$check = $statement->fetchrow();
	$statement->finish();
	
	# Check if registration is complete
	if (defined($check))
	{ return 0; }						# Or return false


	# Database updates
	# If non-authoritative
	if (defined($param->{'JAVA'}))
	{
		#####################
		# Data members
		my $radiojava;					# Formated javascript
		my $radiochat;					# Formatted chat preferences
		my $radioformat;				# Formatted format options
		my $option;					# Options handle
	

		# Link with hash
		$database->GetHashLogin(\%login);

		# Verify options received
		$option      = new Ethereal::Option();
		$radiojava   = $option->FindJavascript($login{'OptJavascript'}, $param->{'JAVA'});
		$radiochat   = $option->FindChatPrefs($login{'OptChatPref'}, $param->{'CHAT'});
		$radioformat = $option->FindFormatting($login{'OptFormat'}, $param->{'FORMAT'});

		# Javascript options
		if ($radiojava ne 'default')
		{
			# Update database entry
			$data->do("UPDATE Puppeteer 
				SET   PuppeteerJavascript=? 
				WHERE PuppeteerLogin=?", {}, 

			 $radiojava, 
			 $param->{'USER'});
		}
	
		# Chatting options
		if ($radiochat ne 'default')
		{
			# Update database entry
			$data->do("UPDATE Puppeteer 
				SET   PuppeteerChatPref=? 
				WHERE PuppeteerLogin=?", {}, 

			 $radiochat, 
			 $param->{'USER'});
		}

		# Formatting options
		if ($radioformat ne 'default')
		{
			# Update database entry
			$data->do("UPDATE Puppeteer 
				SET   PuppeteerFormatting=? 
				WHERE PuppeteerLogin=?", {}, 
			 $radioformat, 
			 $param->{'USER'});
		}
	}

	# New authenthication login
	if ((defined($param->{'PASS'})) || (defined($param->{'COOKIE'})))
	{
		# Update database entry update timestamp
		$data->do("UPDATE Puppeteer 
 		       	SET PuppeteerTimestamp=?, 
       			    PuppeteerHostname=?
       			WHERE PuppeteerLogin=?", {}, 

		 time, 
		 $cgi->remote_host(),
		 $param->{'USER'});

		my $audit = new Ethereal::Audit($database, $cgi, $param);
		$audit->Auth($param->{'USER'});
	} 

	# Cleanup
	$param->Cleanup('CHAT', 
		'COOKIE', 
		'FORMAT', 
		'JAVA', 
		'PASS', 
		$login{'TxtSubmitButton'});

	return 1;						# Return true
}


####################
# Check Legality
#
# Determins the legality of the user using the currently selected realm.  This 
# will also verify if a realm and puppet were selected.    

sub CheckLegality
{
	####################
	# Data members	
	my $self     = shift;					# Reference to self
	my $realm    = shift;					# Realm list

	my $cgi      = $self->{'CGI'};				# Common gateway interface
	my $database = $self->{'HANDLE'};			# Database handle
	my $param    = $self->{'PARAM'};			# Parameter list

	my $access;						# Realm access
	my $message;						# Warning message
	my $redirect;						# Redirect information
	my $table;						# Table handler
	my $verdict;						# To be or not to be

	####################
	# Program area

	# Verification of needed information
	if ((defined($param->{'ROOM'})) && (defined($param->{'CHAR'})))
	{
		# Verification of access
		($access)  = $database->DataGetRealmAccess($param->{'ROOM'});
		($verdict) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});
		
		# Return true if not banned
		# Relaxed realms
		if ((!defined($verdict)) && ($access eq 'relaxed'))
		{ return 1; }

		# Restricted Realms
		elsif ((defined($verdict)) && ($access ne 'relaxed'))
		{ return 1; }

		# Warn of banning
		else
		{ $message = $realm->{'MsgBanned'}; }
	}
	else
	{
		# Warn of missing parameters
		$message = $realm->{'MsgInvalid'};
	}

	# Redirection
	# Retreive inline parameters
	$redirect = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});


	# File header
	print $cgi->header(-type=>'text/html', -refresh=>"35;URL=$realm->{'LnkPortal'}$redirect");


	# HTML Display
	# Header
	$database->DocumentGetHeader();

	# Message Display
	$table = new Ethereal::Table($cgi);
	$table->MakeTop();
	$table->MakeSingle($message);
	$table->MakeBlank();
	$table->MakeSingle("<A HREF=\"$realm->{'LnkPortal'}$redirect\"><CENTER>$realm->{'TagLnkPortal'}</CENTER></A>");
	$table->MakeBottom();

	# Footer
	$database->DocumentGetFooter();

	# Premature exit
	exit;							# Premature exit
}


#####################
# GatherInfo
#
# Sub-routine that will handle the display of information, comes in two flavors
# adminstrative and non-authoritative login.   Selected though alias.

sub GatherInfo
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $choice   = shift;					# Choice on the matter

	my $cgi      = $self->{'CGI'};				# Common gateway interface
	my $database = $self->{'HANDLE'};			# Database handle
	my $param    = $self->{'PARAM'};			# Parameter list
	my $table    = new Ethereal::Table($cgi);		# Table handler

	my %login;						# Login hash


	#####################
	# Program area

	# Link with hash
	$database->GetHashLogin(\%login);

	# Pages lacking headers
	if ($choice > 99)
	{
		print $cgi->header();
		$database->DocumentGetHeader();
	}


	# Print titles
	# Non-authoritative
	if ($choice >= 99)
	{ print "$login{'TagTitleGeneral'}\n<BR>\n"; }

	# Adminstrative
 	elsif ($choice == 1)
	{ print "$login{'TagTitleAdmin'}\n<BR>\n"; }

	# Supervisory
	elsif ($choice == 2)
	{ print "$login{'TagTitleSuper'}\n<BR>\n"; }


	# Start from and embed
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'AUTH', 'True'), "\n";



	# Top of table
	$table->MakeTop();

	# Username and password
	unless (defined($param->{'AUTH'}))
	{
		$table->MakeValid($login{'TagUsername'}, $cgi->textfield('USER', '', 15, 10));
		$table->MakeValid($login{'TagPassword'}, $cgi->password_field('PASS', '', 15, 8));
	}
	else
	{
		$table->MakeInvalid($login{'TagUsername'}, $cgi->textfield('USER', '', 15, 10));
		$table->MakeInvalid($login{'TagPassword'}, $cgi->password_field('PASS', '', 15, 8));
	}

	# Spacer
	$table->MakeBlank();

	# Submit and reset buttons
	$table->MakeValid($cgi->submit($login{'TxtSubmitButton'}), $cgi->reset($login{'TxtResetButton'}));
	$table->MakeBottom();


	# If non admistrative
	if ($choice == 99)
	{
		#####################
		# Data members	
		my $option;					# Option handler
		my $radiojava;					# Formated javascript
		my $radiochat;					# Formatted chat preferences
		my $radioformat;				# Formatted format options

		my @radiojava;					# Javascript options
		my @radiochat;					# Chat options
		my @radioformat;				# Formatting options


		# Split data into arrays
		$option = new Ethereal::Option();
		$option->Split(\@radiojava, $login{'OptJavascript'});
		$option->Split(\@radiochat, $login{'OptChatPref'});
		$option->Split(\@radioformat, $login{'OptFormat'});

		# Format options nicely
		$radiojava   = $cgi->radio_group('JAVA',   \@radiojava,   $radiojava[2],   'true');
		$radiochat   = $cgi->radio_group('CHAT',   \@radiochat,   $radiochat[4],   'true');
		$radioformat = $cgi->radio_group('FORMAT', \@radioformat, $radioformat[3], 'true');

		print "<BR>\n<BR>\n";

		# Table top and title
		$table->MakeTop();
		$table->MakeTertiary($login{'TagOptJavacript'}, $login{'TagOptChatPref'}, $login{'TagOptFormat'});

		# Options and table bottom
		$table->MakeTertiary($radiojava, $radiochat, $radioformat);
		$table->MakeBottom();
	}

	# End of form
	print $cgi->end_form(), "\n";

	# If lacking information
	if ($choice > 99)
	{
		$database->DocumentGetFooter();
	}

}



#####################
# Get Verification
#
# Primary entrance which will lead to GatherInfo or Authenthication depending
# on received parameters.

sub GetVerification
{
	#####################
	# Data members
	my $self     = shift;					# Reference to self
	my $choice   = shift;					# Choice on the matter

	my $cgi      = $self->{'CGI'};				# Common gateway interface
	my $database = $self->{'HANDLE'};			# Database handle
	my $param    = $self->{'PARAM'};			# Parameter list
	my $table    = new Ethereal::Table($cgi);		# Table handler

	my $crypto;						# Crypto handler
	my $cookie;						# System cookie

	#####################
	# Program area
	
	# Verify is necessary components are there
	if ((defined($param->{'USER'})) && ((defined($param->{'PASS'})) || (defined($param->{'CRYPT'}))))
	{
		if ($self->Authenthication($choice))
		{ return 1; }					# Return true
	}
	else
	{
		# Retreive cookie
		$cookie = $cgi->cookie(-name=>'CRYPT');

		if (defined($cookie))
		{
			# Retreive infromation
			$crypto = new Ethereal::Crypto();
			$crypto->DecryptParam($cgi, $cookie, $param);

			# Repeat test
			if ((defined($param->{'USER'})) && (defined($param->{'CRYPT'})))
			{
				# Assign check value
				$param->{'COOKIE'} = $param->{'USER'};

				# Verify authenthication
				if ($self->Authenthication($choice))
				{ return 1; }			# Return true
			}
		}
	}

	# Show login screen
	$self->GatherInfo($choice); 
	return 0;
}

# Alias for administrative
sub GetVerificationAdmin
{ GetVerification(shift, 1); }

# Alias for normal
sub GetVerificationNormal
{ GetVerification(shift, 99); }

# Alis for realm specific
sub GetVerificationRealm
{ GetVerification(shift, 100); }

# Alias for supervisory
sub GetVerificationSuper
{ GetVerification(shift, 2); }

1;								# Return true
