#!/bin/perl
#################################################################################
# Created       : Martin Foster
# Modified      : 12/05/2001
#################################################################################
#
# Board - Messages board functionality for Ethereal-Realms
# Copyright (C) 2000-2002  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 204 - 817, 5th Street NE
#                 Calgary, Alberta
#                 T2E 3W9
#
#################################################################################

use CGI qw(:standard);						# Common gateway interface
use CGI::Carp qw(fatalsToBrowser);				# CGI Error logs
use strict;							# Strict variable enforcement

use Ethereal::Audit;						# Auditing handler
use Ethereal::Database;						# Database handler
use Ethereal::Date;						# Date processing
use Ethereal::Login;						# Login functionality
use Ethereal::Param;						# Parameter control
use Ethereal::Table;						# Table creation


#################################################################################
# Data Members
#################################################################################
my $cgi;							# Common gateway interface handle
my $database;							# Database handle
my $login;							# Login handle
my $param;							# Parameter handle

my %board;							# Board hash

#################################################################################
# Program Area
#################################################################################

	# Initial handles
	$cgi      = new CGI;
	$database = new Ethereal::Database();
	$param    = new Ethereal::Param();

	# Connect and fetch
	$database->Connect();
	$param->GetParam($cgi);

	# Retreive hash
	$database->GetHashBoard(\%board);

	# Login handle
	$login    = new Ethereal::Login($database, $cgi, $param);


	# Handle headers
	HandleAlternate($database, $cgi, $param);

	# Document header
	$database->DocumentGetHeader();

	if ($login->GetVerificationNormal())
	{
		if (RealmSelect($database, $cgi, $param, \%board))
		{
			# Make sure logins are possible
			my ($ban) = $database->DataGetBannedStatus($param->{'USER'}, $param->{'ROOM'});

			# Continue if not banned
			unless (defined($ban))
			{
				# Adjust title
				$board{'TagTitle'} =~ s/REALM/$param->{'ROOM'}/gs;

				# Display title
				print "$board{'TagTitle'}\n";

				# Logic handling
				# Writer
				if (defined($param->{'WRITE'}))
				{ PostWrite($database, $cgi, $param, \%board); }

				# Removal
				if (defined($param->{'PURGE'}))
				{ PostPurge($database, $cgi, $param); }

				# Composer
				if ((defined($param->{'REPLY'})) || (defined($param->{'POST'})))
				{ PostComposer($database, $cgi, $param, \%board); }

				# Posts viewer (default)
				else
				{ PostView($database, $cgi, $param, \%board); }
			}

			# Banned can view
			else
			{
				# Posts viewer
				PostView($database, $cgi, $param, \%board);
			}
		}
	}

	# Document footer
	$database->DocumentGetFooter();	

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Handle Alternates
#
# Will handle how alternate boards are displayed.  Essentially this will simply
# create a redirect header sending it to the alternate board if such a thing
# was specified.

sub HandleAlternate
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list

	my $public;						# Public status
	my $altboard;						# Alternate message board
	

	#####################
	# Program area

	# Verification of acceptance
	if (defined($param->{'ROOM'}))
	{
		# Retreive public status
		($public) = $database->DataGetRealmPublic($param->{'ROOM'});

	
		# Verify existence and nature	
		if ((defined($public)) && ($public ne 'no'))
		{
			# Retreive alternate
			($altboard) = $database->DataGetRealmAltBoard($param->{'ROOM'});		
			

			# Exitence of alternate header
			if (defined($altboard))
			{
				# Retriect header
				print $cgi->redirect($altboard);

				# End script
				exit;
			}
		}
		
		# Private or nonexistent	
		else
		{
			# Dispose of value
			$param->Cleanup('ROOM');
		}
	}

	# Display standard header
	print $cgi->header();
}


#####################
# Realm Selection 
#
# Realms are needed in order to complete operations for the message board.  This
# is for the primary reason that the board grants one topic area for each specific
# realm.  Without a public realm, the board topic will not exist.

sub RealmSelect
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle 

	my $public;						# Public status
	my $realms;						# Composit of realms
	my $table;						# Table handle

	my @realms;						# Realm selection


	#####################
	# Program area

	# Display only if needed
	unless (defined($param->{'ROOM'}))
	{
		# Retreive realm list
		$database->GetListPublicRealms(\@realms);

		# Format line for use
		$realms = $cgi->scrolling_list('ROOM', \@realms, $realms[0], 5);

		# Display Realm Prompt
		# Beginning of form
		print $cgi->start_form();

		# Display Table
		$table = new Ethereal::Table($cgi);

		$table->MakeTop();
		$table->MakeValid($board->{'TagRealm'}, $realms);
		$table->MakeBlank();
		$table->MakeValid($cgi->submit($board->{'TxtSelect'}), $cgi->reset($board->{'TxtReset'}));
		$table->MakeBottom();

		# End of form
		print $param->EmbedNormal($cgi, %{$param});
		print $cgi->end_form(), "\n";

		# Return false
		return 0;
	}


	# True if reached
	return 1;
}

#####################
# Post Viewer 
#
# The viewing sub-routine for the message board.  Of course the specific realm is needed
# which is why that this is called after the ROOM is determined. 

sub PostView 
{
	#####################
	# Data members
	my $database = shift;				# Database handle
	my $cgi      = shift;			 	# CGI handle	
	my $param    = shift;				# Parameter list
	my $board    = shift;				# Board hash

	my $data     = $database->{'HANDLE'};		# Database connection handly

	my $statement;					# Query handle

	my $dtime    = 86400;				# Measurement of a day
	my $time     = time;				# Time
	my $stime;					# Search time

	my $left;					# Rows left of entry
	my $right;					# Rows for entry

	my $choice;					# Choice of the matter
	my $inline;					# Inline parameter list
	my $super;					# Supervisory parameter list
	my $url      = $cgi->url();			# Self referencing link

	my $rows     = 10;				# Complete set of rows
	my $count    = 0;				# Rows retreived
	my $order;					# Order to take on
	my $activity;					# Activity based searching

	my $date     = new Ethereal::Date();		# Time handler
	my $res;					# Results hash


	#####################
	# Program area

	# Determine supervisory
	($choice) = $database->DataGetRoomPrivs($param->{'ROOM'}, $param->{'USER'});


	# Maintain directional state
	$param->{'ORDER'} = (defined($param->{'ORDER'})) ? $param->{'ORDER'} : 'ASC';


	# Generate parameter lines
	# Fucntional
	my $post = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'POST', 'True');
	my $back = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'});

	# Activity based
	my $actv = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ACTV', 'True');

	# Order
	my $desc = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', 'DESC');
	my $asc  = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', 'ASC');

	# Time based
	my $day  = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', $param->{'ORDER'}, 'TIME', '1');
	my $week = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', $param->{'ORDER'}, 'TIME', '7');
	my $mnth = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', $param->{'ORDER'}, 'TIME', '31');
	my $year = $param->EmbedInline($cgi, 'USER', $param->{'USER'}, 'CRYPT', $param->{'CRYPT'}, 'ROOM', $param->{'ROOM'}, 'ORDER', $param->{'ORDER'}, 'TIME', '365');


	# Additional searching
	$stime    = (defined($param->{'TIME'})) ? ($time - ($param->{'TIME'} * $dtime)) : 0;
	$activity = (defined($param->{'ACTV'})) ? 'BoardUpdate DESC,' : '';


	# Prepare and execute
	$statement = $data->prepare("SELECT BoardIDNumber,
			 PuppetName,
			 BoardSubject,
			 BoardTimestamp,
			 BoardMessage,
			 BoardLevel,
			 LevelZero,
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive
			FROM  Board 
			WHERE RealmName=?
			  AND BoardTimestamp > $stime
			ORDER BY $activity
			 LevelZero $param->{ORDER}, 
			 LevelOne,
			 LevelTwo,
			 LevelThree,
			 LevelFour,
			 LevelFive"); 
	$statement->execute($param->{'ROOM'});

	
	# HTML Display
	print "<BR>\n\n<CENTER><TABLE WIDTH=\"95%\" CELLSPACING=\"1\" CELLPADDING=\"4\">\n";

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\"><A HREF=\"$url$post\">$board->{TagPost}</A> - ",
      	      "<A HREF=\"$url$actv\">$board->{TagActv}</A> - ",
      	      "<A HREF=\"$url$asc\">$board->{TagAsc}</A> ",
      	      "<A HREF=\"$url$desc\">$board->{TagDesc}</A> - ",
      	      "<A HREF=\"$url$day\">$board->{TagTimeDay}</A> ",
      	      "<A HREF=\"$url$week\">$board->{TagTimeWeek}</A> ",
      	      "<A HREF=\"$url$mnth\">$board->{TagTimeMonth}</A> ",
      	      "<A HREF=\"$url$year\">$board->{TagTimeYear}</A> - ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></TD>\n",
	      "\t</TR>\n";

	# Retreive and display
	while ($res = $statement->fetchrow_hashref())
	{
		# Increment counter
		$count++;

		# Determine rows
		$right = $rows - $res->{'BoardLevel'};
		$left  = $rows - $right;

		# Generate inline
		$inline  = $param->EmbedInline($cgi, %{$param}, 'REPLY', $res->{'BoardIDNumber'});

		# Kill button
		if (defined($choice))
		{ $super = $param->EmbedInline($cgi, %{$param}, 'PURGE', $res->{'BoardIDNumber'}); }

		# Header
		print "\t<TR>\n";

		# Handling based on level
		if ($res->{'BoardLevel'} != 0)		# Children of threads
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadSubject}$res->{BoardSubject}</FONT><BR>",
			      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadFrom}$res->{PuppetName}",
			      " $board->{TagHeadDate} ", $date->GetDate($res->{'BoardTimestamp'}), " ($res->{BoardIDNumber})</FONT><BR>",
			      "</TD>\n";

			# Kill button
			if (defined($super))
			{
				# Administrative login
				print "\t<TR>\n",
				      "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n",
				      "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorHead}\">",
				      "<A HREF=\"$url$super\">$board->{TagKillThread}</A>",
				      "</TD>\n",
				      "\t</TR>\n";
			}
		}
		else					# Threads parent
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorHead}\">",
		      	      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadSubject}$res->{BoardSubject}</FONT><BR>",
			      "<FONT COLOR=\"$board->{SetColorText}\">$board->{TagHeadFrom}$res->{PuppetName}",
			      " $board->{TagHeadDate} ", $date->GetDate($res->{'BoardTimestamp'}), " ($res->{BoardIDNumber})</FONT><BR>",
			      "</TD>\n";

			# Kill button
			if (defined($super))
			{
				# Administrative login
				print "\t<TR>\n",
				      "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorHead}\">",
				      "<A HREF=\"$url$super\">$board->{TagKillThread}</A>",
				      "</TD>\n",
				      "\t</TR>\n";
			}
		}

		print "\t</TR>\n";


		# Message
		print "\t<TR>\n";

		# Handling based on level
		if ($res->{'BoardLevel'} != 0)		# Children of threads
		{
			print "\t\t<TD COLSPAN=\"$left\">&nbsp;</TD>\n";
			print "\t\t<TD COLSPAN=\"$right\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}
		else					# Threads parent
		{
			print "\t\t<TD COLSPAN=\"$rows\" BGCOLOR=\"$board->{SetColorBody}\">",
			      "<FONT COLOR=\"$board->{SetColorText}\">$res->{BoardMessage}</FONT>",
			      "<BR><A HREF=\"$url$inline\">$board->{TagReply}</A>",
			      "</TD>\n";
		}
	
		print "\t</TR>\n";
	}

	# Status view
	print "\t<TR BGCOLOR=\"$board->{SetColorHead}\">",
	      "\t\t<TD COLSPAN=\"$rows\">$board->{TagCount} $count<BR>",
	      "<A HREF=\"$url$post\">$board->{TagPost}</A> ",
      	      "<A HREF=\"$url$back\">$board->{TagBack}</A></TD>\n",
	      "\t</TR>\n";


	# Spacer row
	print "\t<TR>\n";
	for (my $i; $i < $rows; $i++) { print "\t\t<TD>&nbsp;</TD>\n"; } 

	# End of table
	print "\t</TR>\n</TABLE></CENTER>\n";


	# Close query handle
	$statement->finish();
}


#####################
# Post Composer  
#
# As the name would imply, this subroutine allows for the addition of posts to the 
# existing set of posts.  Provided that there are posts to reply to, one can
# reply under the parent post.

sub PostComposer
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle 

	my $subj;						# Subject of message
	my $table;						# Table handle

	my $puppets;						# Prepared output
	my @puppets;						# List of puppets


	#####################
	# Program area
	
	# Gather information
 	if (defined($param->{'REPLY'}))
	{
		# Prepare and execute
	 	($subj) = $database->DataGet("SELECT BoardSubject 
			FROM Board 
			WHERE BoardIDNumber=?", $param->{'REPLY'});	

		# Adapt subject
		if ($subj !~ /^\(RE\)/)
		{ $subj =~ s/(.+)/(RE) $1/; }

		# Assign ID for next pass
		$param->{'PID'} = $param->{'REPLY'};
	}

	# Cleanup
	$param->Cleanup('REPLY', 'POST');


	# Puppet list 
	# Retreive puppet list
	$database->GetListPuppetYours(\@puppets, $param->{'USER'});

	# Determine puppets
	$puppets = $cgi->popup_menu('PFROM', \@puppets, $puppets[0]);


	# Display area
	# Display title
	print "$board->{'TagTitlePost'}\n";

	# Start of form
	print $cgi->start_form();
	print $param->EmbedNormal($cgi, %{$param}, 'WRITE', 'True'), "\n";

	# Table top
	$table = new Ethereal::Table($cgi);
	$table->MakeTop();

	# Header
	$table->MakeSingle($board->{'TagPostHead'});
	$table->MakeValid($board->{'TagPostFrom'}, $puppets);
	$table->MakeValid($board->{'TagPostSubject'}, $cgi->textfield('PSUBJ', $subj,25,250));

	# Body
	$table->MakeBlank();
	$table->MakeSingle($board->{'TagPostMessage'});
	$table->MakeSingle("<TEXTAREA NAME=\"PTEXT\", ROWS=\"10\", COLS=\"60\", WRAP=\"SOFT\"></TEXTAREA>");


	# Submit and clear
	$table->MakeBlank();
	$table->MakeValid($cgi->submit($board->{'TxtPost'}), $cgi->reset($board->{'TxtReset'}));
	$table->MakeBottom();

	# End form
	print $cgi->end_form(), "\n";
}


#####################
# Post Purge
#
# Handles the purging or killing of post and any children associated with it.   As one can
# see this is useful for giving the message board a more youthful look and cleaning out
# unwanted material.

sub PostPurge
{
	#####################
	# Data members
	my $database = shift;					# Database handle
	my $cgi	     = shift;					# Common gateway interface
	my $param    = shift;					# Parameter list

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $super;						# Supervisory access

	#####################
	# Program Area

	# Audit handle
	my $audit = new Ethereal::Audit($database, $cgi, $param);

	# Superisory access
	($super)  = $database->DataGetRoomPrivs($param->{'ROOM'}, $param->{'USER'});

	# Ensure they are supervisors
	if (defined($super)) 
	{
		# Pruge posts
		$data->do("DELETE FROM Board
			WHERE RealmName=? 
			  AND (LevelZero=? 
			   OR LevelOne=?
			   OR LevelTwo=?
			   OR LevelThree=?
			   OR LevelFive=?)", {},

			 $param->{'ROOM'},
			 $param->{'PURGE'},	
			 $param->{'PURGE'},	
			 $param->{'PURGE'},	
			 $param->{'PURGE'},	
			 $param->{'PURGE'});
	}

	# Auditing table
	$audit->BoardRem($param->{'PURGE'});

	# Cleanup
	$param->Cleanup('PURGE');
}


#####################
# Post Writer  
#
# Handles the eventual cleanup and posting of the post to the threads and tree.  This includes
# language filtering and removal of HTML to keep this clean.
	
sub PostWrite	 						# Database, CGI, Parameter list
{
	#####################
	# Data members
	my $database = shift; 					# Database handle
	my $cgi      = shift;					# CGI handle
	my $param    = shift;					# Parameter list
	my $board    = shift;					# Board hash handle

	my $data     = $database->{'HANDLE'};			# Database connection handle
	my $statement;						# Query statement

	my $puppet;						# Puppet name
	my $puppeteer;						# Puppeteer name

	my $level;						# Level
	my $res;						# Results

	my $id  = 'LAST_INSERT_ID()';				# ID insertion function
	my $url = $cgi->url();					# Location to self

	my @levels;						# Levels and depth
	my %system;						# System hash


	#####################
	# Program area

	# Information handling
	# Retreive post information
	if (defined($param->{'PID'}))
	{
		# Prepare and execute
		$statement = $data->prepare("SELECT PuppetName,
				 BoardLevel,
				 LevelZero,
				 LevelOne,
				 LevelTwo,
				 LevelThree,
				 LevelFour,
				 LevelFive
				FROM  Board
				WHERE BoardIDNumber=?");
		$statement->execute($param->{'PID'});

		# Retreive and adapt
		$res = $statement->fetchrow_hashref();

		# Determine levels
		$levels[0] = $res->{'LevelZero'};
		$levels[1] = ($res->{'LevelOne'}   != $res->{'LevelZero'})  ? $res->{'LevelOne'}   : $id;
		$levels[2] = ($res->{'LevelTwo'}   != $res->{'LevelOne'})   ? $res->{'LevelTwo'}   : $id;
		$levels[3] = ($res->{'LevelThree'} != $res->{'LevelTwo'})   ? $res->{'LevelThree'} : $id;
		$levels[4] = ($res->{'LevelFour'}  != $res->{'LevelThree'}) ? $res->{'LevelFour'}  : $id;
		$levels[5] = ($res->{'LevelFive'}  != $res->{'LevelFour'})  ? $res->{'LevelFive'}  : $id;

		# Determin response level
		$level = $res->{'BoardLevel'} + 1;
		$level = ($level > 5) ? 5 : $level; 

		# Determin puppet name
		$puppet = $res->{'PuppetName'};


		# Finish query
		$statement->finish();
	}
	
	# Parent (root level) posts
	else
	{
		# Assign levels and level for root
		@levels = ($id, $id, $id, $id, $id, $id); 
		$level = 0; 
	}


	# Mold subject and body
	# Strip HTML
	$param->{'PSUBJ'} =~ s/<[^>]*>//gs;
	$param->{'PTEXT'} =~ s/<[^>]*>//gs;

	# Replace carriage returns
	$param->{'PTEXT'} =~ s/\n/<BR>/gs;


	# Database insertion
	$data->do("INSERT INTO Board 
		(BoardIDNumber,
		 PuppetName,
		 RealmName,
		 BoardSubject,
		 BoardTimestamp,
		 BoardMessage,
		 BoardUpdate,
		 BoardLevel,
		 LevelZero,
		 LevelOne,
		 LevelTwo,
		 LevelThree,
		 LevelFour,
		 LevelFive)
		VALUES
		(NULL,?,?,?,?,?,?,?,0,0,0,0,0,0)", {},

	 $param->{'PFROM'},
	 $param->{'ROOM'},
	 $param->{'PSUBJ'},
	 time,
	 $param->{'PTEXT'},
	 time,
	 $level);

	# MySQL kludge
	$data->do("UPDATE Board
		SET LevelZero=$levels[0],
		    LevelOne=$levels[1],
		    LevelTwo=$levels[2],
		    LevelThree=$levels[3],
		    LevelFour=$levels[4],
		    LevelFive=$levels[5]
		WHERE BoardIDNumber=LAST_INSERT_ID()", {});

	# Bump up process
	$data->do("UPDATE Board
		SET BoardUpdate=?
		WHERE LevelZero=?", {}, 

	 time,
	 $levels[0]);



	# Email notification
	if (defined($puppet))
	{
		# Retreive login
		($puppeteer) = $database->DataGetPuppeteerLogin($puppet);


		# Complete if defined
		if (defined($puppeteer))
		{
			# Retreive name and Email
			my ($name)   = $database->DataGetPuppeteerName($puppeteer);
			my ($email)  = $database->DataGetPuppeteerEmail($puppeteer);
			my ($notice) = $database->DocumentGetBoardNotice();

			if (defined($notice))
			{
				# Link with hash
				$database->GetHashSystem(\%system);

 				# Go though all keywords and replace with real value
 				$notice =~ s/FULLNAME/$name/g;
 				$notice =~ s/ORIGIN/$param->{'PFROM'}/g;
				$notice =~ s/REALM/$param->{'ROOM'}/g;
 				$notice =~ s/RETURNEMAIL/$system{'SetInfoContactAddress'}/g;
 				$notice =~ s/SERVERNAME/$system{'SetInfoServer'}/g;
 				$notice =~ s/SUBJECT/$param->{'PSUBJ'}/g;
 				$notice =~ s/URL/$url/g;


				# Define and open sendmail
				open(SENDMAIL, "|$system{'LocSendmail'} -oi -t")
   				   or die("Cannot fork sendmail: $!");

				# Send out invitation
				print SENDMAIL "From: $system{'SetInfoContactName'} <$system{'SetInfoContactAddress'}>\n";
				print SENDMAIL "To: $name <$email>\n";
				print SENDMAIL "Subject: You have a reply on $param->{ROOM} message board from: $param->{'PFROM'} \n\n";
				print SENDMAIL "$notice\n";
				close(SENDMAIL);
			}
		}
	}


	# Cleanup
	$param->Cleanup(
		'PID',
		'PFROM',
		'PSUBJ',
		'PTEXT',
		'WRITE'
	 );
}
